@@ -44,19 +44,14 @@ using Banshee.Query;
 using Banshee.Database;
 
 namespace Banshee.Collection.Database
-{
-    public enum ReloadTrigger {
-        Query,
-        ArtistFilter,
-        AlbumFilter
-    };
-        
+{       
     public class DatabaseTrackListModel : TrackListModel, IExportableModel, 
         ICacheableDatabaseModel, IFilterable, ISortable, ICareAboutView
     {
         private readonly BansheeDbConnection connection;
         private IDatabaseTrackModelProvider provider;
         protected IDatabaseTrackModelCache cache;
+        private Banshee.Sources.DatabaseSource source;
         
         private long count;
 
@@ -72,67 +67,66 @@ namespace Banshee.Collection.Database
         private string join_table, join_fragment, join_primary_key, join_column, condition;
 
         private string query_fragment;
-        private string filter;
-        private string artist_id_filter_query;
-        private string album_id_filter_query;
-
-        private DatabaseArtistListModel artist_model;
-        private DatabaseAlbumListModel album_model;
+        private string user_query;
 
         private int rows_in_view;
         
-        public DatabaseTrackListModel (BansheeDbConnection connection, IDatabaseTrackModelProvider provider)
+        public DatabaseTrackListModel (BansheeDbConnection connection, IDatabaseTrackModelProvider provider, Banshee.Sources.DatabaseSource source)
         {
             this.connection = connection;
             this.provider = provider;
+            this.source = source;
         }
 
         private bool initialized = false;
         public void Initialize (IDatabaseTrackModelCache cache)
-        {
-            Initialize (cache, null, null);
-        }
-
-        public void Initialize (IDatabaseTrackModelCache cache, DatabaseArtistListModel artist_model, DatabaseAlbumListModel album_model)
         {
             if (initialized)
                 return;
 
-            this.artist_model = artist_model;
-            this.album_model = album_model;
-
             initialized = true;
             this.cache = cache;
             cache.AggregatesUpdated += HandleCacheAggregatesUpdated;
-
             GenerateSortQueryPart ();
         }
         
-        private bool have_new_filter = true;
-        private void GenerateFilterQueryPart ()
+        private bool have_new_user_query = true;
+        private void GenerateUserQueryFragment ()
         {
-            if (!have_new_filter)
+            if (!have_new_user_query)
                 return;
 
-            if (String.IsNullOrEmpty (Filter)) {
+            if (String.IsNullOrEmpty (UserQuery)) {
                 query_fragment = null;
+                query_tree = null;
             } else {
-                QueryNode query_tree = UserQueryParser.Parse (Filter, BansheeQuery.FieldSet);
+                query_tree = UserQueryParser.Parse (UserQuery, BansheeQuery.FieldSet);
                 query_fragment = (query_tree == null) ? null : query_tree.ToSql (BansheeQuery.FieldSet);
 
                 if (query_fragment != null && query_fragment.Length == 0) {
                     query_fragment = null;
+                    query_tree = null;
                 }
             }
 
-            have_new_filter = false;
+            have_new_user_query = false;
         }
 
-        private void GenerateSortQueryPart ()
+        private QueryNode query_tree;
+        public QueryNode Query {
+            get { return query_tree; }
+        }
+
+        protected string SortQuery {
+            get { return sort_query; }
+            set { sort_query = value; }
+        }
+
+        protected virtual void GenerateSortQueryPart ()
         {
-            sort_query = (sort_column == null)
+            SortQuery = (SortColumn == null)
                 ? BansheeQuery.GetSort ("Artist", true)
-                : BansheeQuery.GetSort (sort_column.SortKey, sort_column.SortType == SortType.Ascending);
+                : BansheeQuery.GetSort (SortColumn.SortKey, SortColumn.SortType == SortType.Ascending);
         }
 
         public void Sort (ISortableColumn column)
@@ -169,22 +163,28 @@ namespace Banshee.Collection.Database
             OnCleared ();
         }
 
+        public void InvalidateCache ()
+        {
+            if (cache == null) {
+                Log.ErrorFormat ("Called invalidate cache for {0}'s track model, but cache is null", source);
+            } else {
+                cache.ClearManagedCache ();
+                OnReloaded ();
+            }
+        }
+
         private string unfiltered_query;
         protected string UnfilteredQuery {
             get {
                 return unfiltered_query ?? unfiltered_query = String.Format (
                     "FROM {0}{1} WHERE {2} {3}",
-                    provider.From, JoinFragment, provider.Where, ConditionFragment
+                    provider.From, JoinFragment,
+                    String.IsNullOrEmpty (provider.Where) ? "1=1" : provider.Where,
+                    ConditionFragment
                 );
             }
         }
 
-        /*public void UpdateAggregates ()
-        {
-            UpdateUnfilteredAggregates ();
-            UpdateFilteredAggregates ();
-        }*/
-
         private void UpdateUnfilteredAggregates ()
         {
             HyenaSqliteCommand count_command = new HyenaSqliteCommand (String.Format (
@@ -193,45 +193,53 @@ namespace Banshee.Collection.Database
             count = connection.Query<long> (count_command);
         }
 
-        /*private void UpdateFilteredAggregates ()
-        {
-            cache.UpdateAggregates ();
-            filtered_count = cache.Count;
-        }*/
-        
         public override void Reload ()
         {
-            Reload (ReloadTrigger.Query);
+            Reload (null);
         }
 
-        public void Reload (ReloadTrigger trigger)
+        public void Reload (IListModel reloadTrigger)
         {
+            if (cache == null) {
+                Log.WarningFormat ("Called Reload on {0} for source {1} but cache is null;  Did you forget to call AfterInitialized () in your DatabaseSource ctor?",
+                    this, source == null ? "null source!" : source.Name);
+                return;
+            }
+
             lock (this) {
-                bool artist_reloaded = false, album_reloaded = false;
-                GenerateFilterQueryPart ();
+                GenerateUserQueryFragment ();
 
                 UpdateUnfilteredAggregates ();
                 cache.SaveSelection ();
 
-                if (trigger == ReloadTrigger.AlbumFilter) {
-                    ReloadWithFilters ();
+                List<IFilterListModel> reload_models = new List<IFilterListModel> ();
+                bool found = (reloadTrigger == null);
+                foreach (IFilterListModel model in source.FilterModels) {
+                    if (found) {
+                        reload_models.Add (model);
+                    } else if (model == reloadTrigger) {
+                        found = true;
+                    }
+                }
+
+                if (reload_models.Count == 0) {
+                    ReloadWithFilters (true);
                 } else {
-                    ReloadWithoutArtistAlbumFilters ();
+                    ReloadWithoutFilters ();
 
-                    if (artist_model != null && album_model != null) {
-                        if (trigger == ReloadTrigger.Query) {
-                            artist_reloaded = true;
-                            artist_model.Reload (false);
-                        }
-
-                        album_reloaded = true;
-                        album_model.Reload (false);
-
-                        // Unless both artist/album selections are "all" (eg unfiltered), reload
-                        // the track model again with the artist/album filters now in place.
-                        if (!artist_model.Selection.AllSelected || !album_model.Selection.AllSelected) {
-                            ReloadWithFilters ();
-                        }
+                    foreach (IFilterListModel model in reload_models) {
+                        model.Reload (false);
+                    }
+                    
+                    bool have_filters = false;
+                    foreach (IFilterListModel model in source.FilterModels) {
+                        have_filters |= !model.Selection.AllSelected;
+                    }
+                    
+                    // Unless both artist/album selections are "all" (eg unfiltered), reload
+                    // the track model again with the artist/album filters now in place.
+                    if (have_filters) {
+                        ReloadWithFilters (true);
                     }
                 }
 
@@ -243,21 +251,34 @@ namespace Banshee.Collection.Database
                 OnReloaded ();
 
                 // Trigger these after the track list, b/c visually it's more important for it to update first
-                if (artist_reloaded)
-                    artist_model.RaiseReloaded ();
-
-                if (album_reloaded)
-                    album_model.RaiseReloaded ();
+                foreach (IFilterListModel model in reload_models) {
+                    model.RaiseReloaded ();
+                }
             }
         }
 
-        private void ReloadWithoutArtistAlbumFilters ()
+        private void ReloadWithoutFilters ()
+        {
+            ReloadWithFilters (false);
+        }
+
+        private void ReloadWithFilters (bool with_filters)
         {
             StringBuilder qb = new StringBuilder ();
             qb.Append (UnfilteredQuery);
-
+            
+            if (with_filters) {
+                foreach (IFilterListModel model in source.FilterModels) {
+                    string filter = model.GetSqlFilter ();
+                    if (filter != null) {
+                        qb.Append (" AND ");
+                        qb.Append (filter);
+                    }
+                }
+            }
+            
             if (query_fragment != null) {
-                qb.Append ("AND ");
+                qb.Append (" AND ");
                 qb.Append (query_fragment);
             }
             
@@ -265,49 +286,19 @@ namespace Banshee.Collection.Database
                 qb.Append (" ORDER BY ");
                 qb.Append (sort_query);
             }
-                
-            reload_fragment = qb.ToString ();
-
-            cache.Reload ();
-        }
-
-        private void ReloadWithFilters ()
-        {
-            StringBuilder qb = new StringBuilder ();
-            qb.Append (UnfilteredQuery);
-
-            ArtistInfoFilter = artist_model.SelectedItems;
-            AlbumInfoFilter = album_model.SelectedItems;
-
-            if (artist_id_filter_query != null) {
-                qb.Append ("AND ");
-                qb.Append (artist_id_filter_query);
-            }
-                    
-            if (album_id_filter_query != null) {
-                qb.Append ("AND ");
-                qb.Append (album_id_filter_query);
-            }
             
-            if (query_fragment != null) {
-                qb.Append ("AND ");
-                qb.Append (query_fragment);
-            }
-            
-            if (sort_query != null) {
-                qb.Append (" ORDER BY ");
-                qb.Append (sort_query);
-            }
-                
             reload_fragment = qb.ToString ();
-
             cache.Reload ();
         }
 
         public override int IndexOf (TrackInfo track)
         {
             DatabaseTrackInfo db_track = track as DatabaseTrackInfo;
-            return (int) (db_track == null ? -1 : cache.IndexOf ((int)db_track.TrackId));
+            if (db_track == null || db_track.CacheModelId != CacheId) {
+                return -1;
+            }
+
+            return (int) cache.IndexOf ((int)db_track.CacheEntryId);
         }
 
         private DateTime random_began_at = DateTime.MinValue;
@@ -358,12 +349,12 @@ namespace Banshee.Collection.Database
             get { return (int) count; }
         }
 
-        public string Filter {
-            get { return filter; }
+        public string UserQuery {
+            get { return user_query; }
             set { 
                 lock (this) {
-                    filter = value; 
-                    have_new_filter = true;
+                    user_query = value; 
+                    have_new_user_query = true;
                 }
             }
         }
@@ -398,10 +389,14 @@ namespace Banshee.Collection.Database
             get { return join_column; }
             set { join_column = value; }
         }
+        
+        public void AddCondition (string part)
+        {
+            condition = condition == null ? part : String.Format ("{0} AND {1}", condition, part);
+        }
 
         public string Condition {
             get { return condition; }
-            set { condition = value; }
         }
 
         public string ConditionFragment {
@@ -411,69 +406,11 @@ namespace Banshee.Collection.Database
         private string PrefixCondition (string prefix)
         {
             string condition = Condition;
-            if (condition == null || condition == String.Empty)
-                return String.Empty;
-            else
-                return String.Format (" {0} {1} ", prefix, condition);
+            return String.IsNullOrEmpty (condition)
+                ? String.Empty
+                : String.Format (" {0} {1} ", prefix, condition);
         }
 
-        public override IEnumerable<ArtistInfo> ArtistInfoFilter {
-            set {
-                ModelHelper.BuildIdFilter<ArtistInfo> (value, "CoreTracks.ArtistID", artist_id_filter_query,
-                    delegate (ArtistInfo artist) {
-                        if (!(artist is DatabaseArtistInfo)) {
-                            return null;
-                        }
-                        
-                        return ((DatabaseArtistInfo)artist).DbId.ToString ();
-                    },
-                
-                    delegate (string new_filter) {
-                        artist_id_filter_query = new_filter;
-                    }
-                );
-            }
-        }
-        
-        public override IEnumerable<AlbumInfo> AlbumInfoFilter {
-            set { 
-                ModelHelper.BuildIdFilter<AlbumInfo> (value, "CoreTracks.AlbumID", album_id_filter_query,
-                    delegate (AlbumInfo album) {
-                        if (!(album is DatabaseAlbumInfo)) {
-                            return null;
-                        }
-                        
-                        return ((DatabaseAlbumInfo)album).DbId.ToString ();
-                    },
-                
-                    delegate (string new_filter) {
-                        album_id_filter_query = new_filter;
-                    }
-                );
-            }
-        }
-
-        public override void ClearArtistAlbumFilters ()
-        {
-            artist_id_filter_query = null;
-            album_id_filter_query = null;
-            Reload ();
-        }
-
-        /*private HyenaSqliteCommand check_artists_command = new HyenaSqliteCommand (
-            "SELECT ItemID FROM CoreCache WHERE ModelID = ? AND ItemID NOT IN (SELECT ArtistID FROM CoreArtists)"
-        );
-
-        private HyenaSqliteCommand check_albums_command = new HyenaSqliteCommand (
-            "SELECT ItemID FROM CoreCache WHERE ModelID = ? AND ItemID NOT IN (SELECT AlbumID FROM CoreAlbums)"
-        );*/
-
-        /*public void CheckFilters ()
-        {
-            if (track_model.Artist
-            if (ServiceManager.DbConnection.Query<int> (
-        }*/
-
         public int CacheId {
             get { return (int) cache.CacheId; }
         }
