@@ -49,9 +49,11 @@ namespace Banshee.Sources
         public int Position;
     }
     
-    public class SourceManager : ISourceManager, IDisposable
+    public class SourceManager : ISourceManager, IInitializeService, IRequiredService, IDisposable
     {
         private List<Source> sources = new List<Source>();
+        private Dictionary<string, Source> extension_sources = new Dictionary<string, Source> ();
+        
         private Source active_source;
         private Source default_source;
         private MusicLibrarySource music_library;
@@ -61,19 +63,68 @@ namespace Banshee.Sources
         public event SourceAddedHandler SourceAdded;
         public event SourceEventHandler SourceRemoved;
         public event SourceEventHandler ActiveSourceChanged;
-        
+
+        public void Initialize ()
+        {
+            // TODO should add library sources here, but requires changing quite a few
+            // things that depend on being loaded before the music library is added.
+            //AddSource (music_library = new MusicLibrarySource (), true);
+            //AddSource (video_library = new VideoLibrarySource (), false);
+        }
+
         internal void LoadExtensionSources ()
         {
-            foreach (TypeExtensionNode node in AddinManager.GetExtensionNodes ("/Banshee/SourceManager/Source")) {
-                node.CreateInstance (typeof (ISource));
+            lock (this) {
+                AddinManager.AddExtensionNodeHandler ("/Banshee/SourceManager/Source", OnExtensionChanged);
             }
         }
         
         public void Dispose ()
         {
-            foreach (Source source in sources) {
-                if (source is IDisposable) {
-                    ((IDisposable)source).Dispose ();
+            lock (this) {
+                try {
+                    AddinManager.RemoveExtensionNodeHandler ("/Banshee/SourceManager/Source", OnExtensionChanged);
+                } catch {}
+
+                active_source = null;
+                default_source = null;
+                music_library = null;
+                video_library = null;
+
+                // Do dispose extension sources
+                foreach (Source source in extension_sources.Values) {
+                    RemoveSource (source, true);
+                }
+
+                // But do not dispose non-extension sources
+                while (sources.Count > 0) {
+                    RemoveSource (sources[0], false);
+                }
+                
+                sources.Clear ();
+                extension_sources.Clear ();
+            }
+        }
+        
+        private void OnExtensionChanged (object o, ExtensionNodeEventArgs args) 
+        {
+            lock (this) {
+                TypeExtensionNode node = (TypeExtensionNode)args.ExtensionNode;
+                
+                if (args.Change == ExtensionChange.Add && !extension_sources.ContainsKey (node.Id)) {
+                    Source source = (Source)node.CreateInstance ();
+                    extension_sources.Add (node.Id, source);
+                    bool add_source = true;
+                    if (source.Properties.Contains ("AutoAddSource")) {
+                        add_source = source.Properties.GetBoolean ("AutoAddSource");
+                    }
+                    if (add_source) {
+                        AddSource (source);
+                    }
+                } else if (args.Change == ExtensionChange.Remove && extension_sources.ContainsKey (node.Id)) {
+                    Source source = extension_sources[node.Id];
+                    extension_sources.Remove (node.Id);
+                    RemoveSource (source, true);
                 }
             }
         }
@@ -113,7 +164,7 @@ namespace Banshee.Sources
             } else if (source is VideoLibrarySource) {
                 video_library = source as VideoLibrarySource;
             }
-            
+
             ServiceManager.DBusServiceManager.RegisterObject(source);
             
             foreach(Source child_source in source.Children) {
@@ -125,7 +176,12 @@ namespace Banshee.Sources
             }
         }
         
-        public void RemoveSource(Source source)
+        public void RemoveSource (Source source)
+        {
+            RemoveSource (source, false);
+        }
+
+        public void RemoveSource (Source source, bool recursivelyDispose)
         {
             if(source == null || !ContainsSource (source)) {
                 return;
@@ -142,7 +198,14 @@ namespace Banshee.Sources
             sources.Remove(source);
 
             foreach(Source child_source in source.Children) {
-                RemoveSource(child_source);
+                RemoveSource (child_source, recursivelyDispose);
+            }
+
+            if (recursivelyDispose) {
+                IDisposable disposable = source as IDisposable;
+                if (disposable != null) {
+                    disposable.Dispose ();
+                }
             }
 
             if(source == active_source) {
@@ -249,6 +312,10 @@ namespace Banshee.Sources
         
         public void SetActiveSource(Source source, bool notify)
         {
+            if(source == null || active_source == source) {
+                return;
+            }
+            
             if(active_source != null) {
                 active_source.Deactivate();
             }
