@@ -29,9 +29,10 @@
 #pragma warning disable 0612
 
 using System;
+using System.Data;
+using System.Collections.Generic;
 using Gtk;
 using Glade;
-using System.Collections.Generic;
 using Mono.Unix;
 
 using Banshee.Base;
@@ -44,6 +45,7 @@ using Banshee.Sources;
 using Banshee.Collection.Database;
 
 using Hyena.Gui;
+using Hyena.Widgets;
 
 namespace Banshee.Gui.Dialogs
 {
@@ -74,18 +76,18 @@ namespace Banshee.Gui.Dialogs
         public int SampleRate;
         public long FileSize;
     
-        public EditorTrack(TrackInfo track)
+        public EditorTrack (TrackInfo track)
         {
             this.track = track;
-            Revert();
+            Revert ();
         }
         
-        public void Revert()
+        public void Revert ()
         {
-            ArtistName = track.ArtistName == null ? String.Empty : track.ArtistName;
-            AlbumTitle = track.AlbumTitle == null ? String.Empty : track.AlbumTitle;
-            TrackTitle = track.TrackTitle == null ? String.Empty : track.TrackTitle;
-            Genre = track.Genre == null ? String.Empty : track.Genre;
+            ArtistName = track.ArtistName ?? String.Empty;
+            AlbumTitle = track.AlbumTitle ?? String.Empty;
+            TrackTitle = track.TrackTitle ?? String.Empty;
+            Genre = track.Genre ?? String.Empty;
             Disc = track.Disc;
             TrackNumber = track.TrackNumber;
             TrackCount = track.TrackCount;
@@ -94,7 +96,7 @@ namespace Banshee.Gui.Dialogs
             Rating = track.Rating;
         }
         
-        public void Save()
+        public void Save ()
         {
             track.ArtistName = ArtistName;
             track.AlbumTitle = AlbumTitle;
@@ -152,30 +154,30 @@ namespace Banshee.Gui.Dialogs
         [Widget] private CheckButton CopyCoverArt;
         [Widget] private Box RatingContainer;
         
-        private Tooltips tips = new Tooltips();
-        private RatingEntry rating_entry = new RatingEntry();
-        private List<EntryUndoAdapter> entry_undo_adapters = new List<EntryUndoAdapter>();
+        private Tooltips tips = new Tooltips ();
+        private RatingEntry rating_entry = new RatingEntry ();
+        private List<EntryUndoAdapter> entry_undo_adapters = new List<EntryUndoAdapter> ();
         
-        private List<EditorTrack> TrackSet = new List<EditorTrack>();
+        private List<EditorTrack> TrackSet = new List<EditorTrack> ();
         private int currentIndex = 0;
         private bool first_load = false;
 
         public event EventHandler Saved;
 
-        public TrackEditor(IEnumerable<TrackInfo> selection) : base("TrackEditorWindow")
+        public TrackEditor (IEnumerable<TrackInfo> selection) : base ("TrackEditorWindow")
         {
-            if(selection == null) {
+            if (selection == null) {
                 return;
             }
         
-            foreach(TrackInfo track in selection) {
+            foreach (TrackInfo track in selection) {
                 if (track != null)
-                    TrackSet.Add(new EditorTrack(track));
+                    TrackSet.Add (new EditorTrack (track));
             }
             
-            rating_entry.Show();
+            rating_entry.Show ();
             (Glade["RatingLabel"] as Label).MnemonicWidget = rating_entry;
-            RatingContainer.PackStart(rating_entry, false, false, 0);
+            RatingContainer.PackStart (rating_entry, false, false, 0);
 
             TrackNumberIterator.ExposeEvent += OnTrackNumberIteratorExpose;
 
@@ -204,14 +206,18 @@ namespace Banshee.Gui.Dialogs
             Genre.Entry.Changed += OnValueEdited;
             rating_entry.Changed += OnValueEdited;
             
-            ListStore genre_model = new ListStore(typeof(string));
+            ListStore genre_model = new ListStore (typeof (string));
             Genre.Model = genre_model;
             Genre.TextColumn = 0;
             
-            // FIXME merge
-            /*foreach(string genre in Globals.Library.GetGenreList()) {
-                genre_model.AppendValues(genre);            
-            }*/
+            IDataReader reader = ServiceManager.DbConnection.Query (
+                "SELECT DISTINCT Genre FROM CoreTracks ORDER BY Genre");
+            while (reader != null && reader.Read ()) {
+                string genre = reader[0] as string;
+                if (!String.IsNullOrEmpty (genre)) {
+                    genre_model.AppendValues (genre);
+                }
+            }
             
             Next.Visible = TrackSet.Count > 1;
             Previous.Visible = TrackSet.Count > 1;
@@ -228,35 +234,35 @@ namespace Banshee.Gui.Dialogs
             EnterNextTitle.Visible = TrackSet.Count > 1;
             Glade["SyncAllAlignment"].Visible = TrackSet.Count > 1;
             
-            EditorNotebook.RemovePage(1);
+            EditorNotebook.RemovePage (1);
 
-            tips.SetTip(TrackNumberIterator, Catalog.GetString("Automatically set all track numbers in increasing order"), "track iterator");
-            tips.SetTip(TrackCountSync, Catalog.GetString("Set all track counts to this value"), "track counts");
-            tips.SetTip(ArtistSync, Catalog.GetString("Set all artists to this value"), "artists");
-            tips.SetTip(AlbumSync, Catalog.GetString("Set all albums to this value"), "albums");
-            tips.SetTip(DiscSync, Catalog.GetString("Set all disc numbers to this value"), "discs");
-            tips.SetTip(GenreSync, Catalog.GetString("Set all genres to this value"), "genres"); 
-            tips.SetTip(YearSync, Catalog.GetString("Set all years to this value"), "years");
-            tips.SetTip(SyncAll, Catalog.GetString("Apply the values of this track set for the Artist, Album Title, Genre, Track count, Year, and Rating fields to the rest of the selected tracks in this editor."), "all");
-            tips.SetTip(RatingSync, Catalog.GetString("Set all ratings to this value"), "ratings");
+            tips.SetTip (TrackNumberIterator, Catalog.GetString ("Automatically set all track numbers in increasing order"), "track iterator");
+            tips.SetTip (TrackCountSync, Catalog.GetString ("Set all track counts to this value"), "track counts");
+            tips.SetTip (ArtistSync, Catalog.GetString ("Set all artists to this value"), "artists");
+            tips.SetTip (AlbumSync, Catalog.GetString ("Set all albums to this value"), "albums");
+            tips.SetTip (DiscSync, Catalog.GetString ("Set all disc numbers to this value"), "discs");
+            tips.SetTip (GenreSync, Catalog.GetString ("Set all genres to this value"), "genres"); 
+            tips.SetTip (YearSync, Catalog.GetString ("Set all years to this value"), "years");
+            tips.SetTip (SyncAll, Catalog.GetString ("Apply the values of this track set for the Artist, Album Title, Genre, Track count, Year, and Rating fields to the rest of the selected tracks in this editor."), "all");
+            tips.SetTip (RatingSync, Catalog.GetString ("Set all ratings to this value"), "ratings");
             
-            LoadTrack(0);
+            LoadTrack (0);
             
-            foreach(Entry entry in new Entry [] { Artist, Album, Title, Year, Genre.Entry }) {
-                entry_undo_adapters.Add(new EntryUndoAdapter(entry));
+            foreach (Entry entry in new Entry [] { Artist, Album, Title, Year, Genre.Entry }) {
+                entry_undo_adapters.Add (new EntryUndoAdapter (entry));
             }
             
-            Window.Show();
+            Window.Show ();
         }
         
-        private void OnTrackNumberIteratorExpose(object o, ExposeEventArgs args)
+        private void OnTrackNumberIteratorExpose (object o, ExposeEventArgs args)
         {
             Gdk.Rectangle alloc = TrackNumberIterator.Allocation;
-            Gdk.GC gc = TrackNumberIterator.Style.DarkGC(StateType.Normal);
+            Gdk.GC gc = TrackNumberIterator.Style.DarkGC (StateType.Normal);
             Gdk.Drawable drawable = TrackNumberIterator.GdkWindow;
             
-            int x_pad = (int)((double)alloc.Width * 0.15);
-            int y_pad = (int)((double)alloc.Height * 0.15);
+            int x_pad = (int) ((double)alloc.Width * 0.15);
+            int y_pad = (int) ((double)alloc.Height * 0.15);
             
             int left_x = alloc.X + x_pad;
             int top_y = alloc.Y + y_pad;
@@ -264,23 +270,23 @@ namespace Banshee.Gui.Dialogs
             int mid_y = alloc.Y + (alloc.Height / 2);
             int bottom_y = (alloc.Y + alloc.Height) - y_pad;
             
-            drawable.DrawLine(gc, left_x, top_y, mid_x, top_y);
-            drawable.DrawLine(gc, mid_x + 1, top_y + 1, mid_x + 1, mid_y - 15);
+            drawable.DrawLine (gc, left_x, top_y, mid_x, top_y);
+            drawable.DrawLine (gc, mid_x + 1, top_y + 1, mid_x + 1, mid_y - 15);
             
-            drawable.DrawLine(gc, left_x, bottom_y, mid_x, bottom_y);
-            drawable.DrawLine(gc, mid_x + 1, bottom_y - 1, mid_x + 1, mid_y + 13);
+            drawable.DrawLine (gc, left_x, bottom_y, mid_x, bottom_y);
+            drawable.DrawLine (gc, mid_x + 1, bottom_y - 1, mid_x + 1, mid_y + 13);
         }
         
-        private void LoadTrack(int index)
+        private void LoadTrack (int index)
         {
-            if(index < 0 || index >= TrackSet.Count) {
+            if (index < 0 || index >= TrackSet.Count) {
                 return;
             }
                 
-            SetCoverImage(null);
+            SetCoverImage (null);
             
-            foreach(EntryUndoAdapter undo_adapter in entry_undo_adapters) {
-                undo_adapter.UndoManager.Clear();
+            foreach (EntryUndoAdapter undo_adapter in entry_undo_adapters) {
+                undo_adapter.UndoManager.Clear ();
             }
                 
             EditorTrack track = TrackSet[index] as EditorTrack;
@@ -288,7 +294,7 @@ namespace Banshee.Gui.Dialogs
             TrackNumber.Value = track.TrackNumber;
             TrackCount.Value = track.TrackCount;
             Disc.Value = track.Disc;
-            Year.Text = track.Year.ToString();
+            Year.Text = track.Year.ToString ();
             rating_entry.Value = (int)track.Rating;
         
             (Glade["Artist"] as Entry).Text = track.ArtistName;
@@ -296,15 +302,15 @@ namespace Banshee.Gui.Dialogs
             (Glade["Title"] as Entry).Text = track.TrackTitle;
             (Glade["Genre"] as ComboBoxEntry).Entry.Text = track.Genre;
             
-            (Glade["DurationLabel"] as Label).Text = String.Format("{0}:{1}", 
-                track.Track.Duration.Minutes, (track.Track.Duration.Seconds).ToString("00"));
-            (Glade["PlayCountLabel"] as Label).Text = track.Track.PlayCount.ToString();
+            (Glade["DurationLabel"] as Label).Text = String.Format ("{0}:{1}", 
+                track.Track.Duration.Minutes, (track.Track.Duration.Seconds).ToString ("00"));
+            (Glade["PlayCountLabel"] as Label).Text = track.Track.PlayCount.ToString ();
             (Glade["LastPlayedLabel"] as Label).Text = track.Track.LastPlayed == DateTime.MinValue ?
-                Catalog.GetString("Never played") : track.Track.LastPlayed.ToString();
+                Catalog.GetString ("Never played") : track.Track.LastPlayed.ToString ();
             (Glade["ImportedLabel"] as Label).Text = track.Track.DateAdded == DateTime.MinValue ?
-                Catalog.GetString("Unknown") : track.Track.DateAdded.ToString();
+                Catalog.GetString ("Unknown") : track.Track.DateAdded.ToString ();
                 
-            if(first_load) {
+            if (first_load) {
                 EmbedCoverArt.Active = true;
                 CopyCoverArt.Active = true;
                 first_load = false;
@@ -314,34 +320,35 @@ namespace Banshee.Gui.Dialogs
             }
             
             Window.Title = TrackSet.Count > 1 
-                ? String.Format(Catalog.GetString("Editing item {0} of {1}"), index + 1, TrackSet.Count)
-                : String.Format(Catalog.GetString("Editing {0}"), track.TrackTitle);
+                ? String.Format (Catalog.GetString ("Editing item {0} of {1}"), index + 1, TrackSet.Count)
+                : String.Format (Catalog.GetString ("Editing {0}"), track.TrackTitle);
        
-            if(track.Uri.IsLocalPath) {
-                Uri.Text = System.IO.Path.GetFileName(track.Uri.LocalPath);
-                Location.Text = System.IO.Path.GetDirectoryName(track.Uri.LocalPath);
+            if (track.Uri.IsLocalPath) {
+                Uri.Text = System.IO.Path.GetFileName (track.Uri.LocalPath);
+                Location.Text = System.IO.Path.GetDirectoryName (track.Uri.LocalPath);
             } else {
-                Uri.Text = track.Uri.ToString();
+                Uri.Text = track.Uri.ToString ();
                 Location.Text = String.Empty;
             }
             
-            FileSize.Text = Catalog.GetString("Unknown");
+            FileSize.Text = Catalog.GetString ("Unknown");
                 
             // FIXME merge
-            //if(!(track.Track is AudioCdTrackInfo) && !track.ProcessedStream) {
-            if(!track.ProcessedStream) {
+            //if (!(track.Track is AudioCdTrackInfo) && !track.ProcessedStream) {
+            if (!track.ProcessedStream) {
                 track.ProcessedStream = true;
                 
-                if(track.Uri.Scheme == System.Uri.UriSchemeFile) {
+                if (track.Uri.Scheme == System.Uri.UriSchemeFile) {
+                    // TODO have this use the TrackInfo's FileSize
                     try {
-                        System.IO.FileInfo info = new System.IO.FileInfo(track.Uri.LocalPath);
+                        System.IO.FileInfo info = new System.IO.FileInfo (track.Uri.LocalPath);
                         track.FileSize = info.Length;
                     } catch {
                     }
                 }
                 
                 try {
-                    TagLib.File file = StreamTagger.ProcessUri(track.Uri);
+                    TagLib.File file = StreamTagger.ProcessUri (track.Uri);
                 
                     /*try {
                         TagLib.IPicture [] pictures = file.Tag.Pictures;
@@ -370,25 +377,25 @@ namespace Banshee.Gui.Dialogs
                     track.Bitrate = file.Properties.AudioBitrate;
                     track.SampleRate = file.Properties.AudioSampleRate;
                     track.Channels = file.Properties.AudioChannels;
-                } catch(Exception) {
+                } catch (Exception) {
                     track.Bitrate = -1;
                 }
             } 
             
-            if(track.ProcessedStream) {
-                FileSize.Text = String.Format("{0:0.0} MB", (double)track.FileSize / 1024.0 / 1024.0);
+            if (track.ProcessedStream) {
+                FileSize.Text = String.Format ("{0:0.0} MB", (double)track.FileSize / 1024.0 / 1024.0);
                 
-                if(track.Bitrate > 0) {
-                    BitRate.Text = String.Format("{0} kbps", track.Bitrate);
-                    SampleRate.Text = String.Format("{0} Hz", track.SampleRate);
-                    Channels.Text = String.Format("{0}", track.Channels);
+                if (track.Bitrate > 0) {
+                    BitRate.Text = String.Format ("{0} kbps", track.Bitrate);
+                    SampleRate.Text = String.Format ("{0} Hz", track.SampleRate);
+                    Channels.Text = String.Format ("{0}", track.Channels);
                 } else {
-                    BitRate.Text = String.Format("{0} kbps", track.Bitrate);
-                    SampleRate.Text = String.Format("{0} Hz", track.SampleRate);
-                    Channels.Text = String.Format("{0}", track.Channels);
+                    BitRate.Text = String.Format ("{0} kbps", track.Bitrate);
+                    SampleRate.Text = String.Format ("{0} Hz", track.SampleRate);
+                    Channels.Text = String.Format ("{0}", track.Channels);
                 }
                 
-                SetCoverImage(track.CoverArtThumbnail);
+                SetCoverImage (track.CoverArtThumbnail);
             }
             
             Previous.Sensitive = index > 0;
@@ -396,33 +403,33 @@ namespace Banshee.Gui.Dialogs
             EnterNextTitle.Sensitive = Next.Sensitive;
         }
         
-        private static Gdk.Pixbuf no_cover_image = Gdk.Pixbuf.LoadFromResource("browser-album-cover.png");
+        private static Gdk.Pixbuf no_cover_image = Gdk.Pixbuf.LoadFromResource ("browser-album-cover.png");
         
-        private void SetCoverImage(Gdk.Pixbuf pixbuf)
+        private void SetCoverImage (Gdk.Pixbuf pixbuf)
         {
-            if(pixbuf == null) {
+            if (pixbuf == null) {
                 CoverImage.Pixbuf = no_cover_image;
             } else {
                 CoverImage.Pixbuf = pixbuf;
             }
         }
         
-        private void OnPreviousClicked(object o, EventArgs args)
+        private void OnPreviousClicked (object o, EventArgs args)
         {
-            UpdateCurrent();
-            LoadTrack(--currentIndex);
+            UpdateCurrent ();
+            LoadTrack (--currentIndex);
         }
         
-        private void OnNextClicked(object o, EventArgs args)
+        private void OnNextClicked (object o, EventArgs args)
         {
-            UpdateCurrent();
-            LoadTrack(++currentIndex);
+            UpdateCurrent ();
+            LoadTrack (++currentIndex);
         }
 
-        private void OnTrackNumberIteratorClicked(object o, EventArgs args)
+        private void OnTrackNumberIteratorClicked (object o, EventArgs args)
         {
             int i = 1;
-            foreach(EditorTrack track in TrackSet) {
+            foreach (EditorTrack track in TrackSet) {
                 track.TrackNumber = i++;
                 track.TrackCount = TrackSet.Count;
             }
@@ -432,130 +439,130 @@ namespace Banshee.Gui.Dialogs
             TrackCount.Value = current_track.TrackCount;
         }
         
-        private void OnValueEdited(object o, EventArgs args)
+        private void OnValueEdited (object o, EventArgs args)
         {
-            if(currentIndex < 0 || currentIndex >= TrackSet.Count) {
+            if (currentIndex < 0 || currentIndex >= TrackSet.Count) {
                 return;
             }
         }
         
-        private void OnTrackCountSyncClicked(object o, EventArgs args)
+        private void OnTrackCountSyncClicked (object o, EventArgs args)
         {
-            foreach(EditorTrack track in TrackSet) {
+            foreach (EditorTrack track in TrackSet) {
                 track.TrackCount = (int)TrackCount.Value;
             }
         }
         
-        private void OnRatingSyncClicked(object o, EventArgs args)
+        private void OnRatingSyncClicked (object o, EventArgs args)
         {
-            foreach(EditorTrack track in TrackSet) {
+            foreach (EditorTrack track in TrackSet) {
                 track.Rating = rating_entry.Value;
             }
         }
         
-        private void OnYearSyncClicked(object o, EventArgs args)
+        private void OnYearSyncClicked (object o, EventArgs args)
         {
-            foreach(EditorTrack track in TrackSet) {
+            foreach (EditorTrack track in TrackSet) {
                 try {
-                    track.Year = Convert.ToInt32(Year.Text);
+                    track.Year = Convert.ToInt32 (Year.Text);
                 } catch {
                     track.Year = 0;
                 }
             }
         }
 
-        private void OnDiscSyncClicked(object o, EventArgs args)
+        private void OnDiscSyncClicked (object o, EventArgs args)
         {
-            foreach(EditorTrack track in TrackSet) {
+            foreach (EditorTrack track in TrackSet) {
                 track.Disc = (int)Disc.Value;
             }
         }
         
-        private void OnArtistSyncClicked(object o, EventArgs args)
+        private void OnArtistSyncClicked (object o, EventArgs args)
         {
-            foreach(EditorTrack track in TrackSet) {
+            foreach (EditorTrack track in TrackSet) {
                 track.ArtistName = Artist.Text;
             }
         }
 
-        private void OnAlbumSyncClicked(object o, EventArgs args)
+        private void OnAlbumSyncClicked (object o, EventArgs args)
         {
-            foreach(EditorTrack track in TrackSet) {
+            foreach (EditorTrack track in TrackSet) {
                 track.AlbumTitle = Album.Text;
             }
         }
         
-        private void OnSyncAllClicked(object o, EventArgs args)
+        private void OnSyncAllClicked (object o, EventArgs args)
         {
-            OnTrackCountSyncClicked(o, args);
-            OnGenreSyncClicked(o, args);
-            OnDiscSyncClicked(o, args);
-            OnAlbumSyncClicked(o, args);
-            OnArtistSyncClicked(o, args);
-            OnYearSyncClicked(o, args);
-            OnRatingSyncClicked(o, args);
+            OnTrackCountSyncClicked (o, args);
+            OnGenreSyncClicked (o, args);
+            OnDiscSyncClicked (o, args);
+            OnAlbumSyncClicked (o, args);
+            OnArtistSyncClicked (o, args);
+            OnYearSyncClicked (o, args);
+            OnRatingSyncClicked (o, args);
         }
         
-        private void OnGenreSyncClicked(object o, EventArgs args)
+        private void OnGenreSyncClicked (object o, EventArgs args)
         {
-            foreach(EditorTrack track in TrackSet) {
+            foreach (EditorTrack track in TrackSet) {
                 track.Genre = Genre.Entry.Text;
             }
         }
         
-        private void OnEnterNextTitleClicked(object o, EventArgs args)
+        private void OnEnterNextTitleClicked (object o, EventArgs args)
         {
-            OnNextClicked(o, args);
+            OnNextClicked (o, args);
             Title.HasFocus = true;
-            Title.SelectRegion(0, Title.Text.Length);
+            Title.SelectRegion (0, Title.Text.Length);
         }
         
-        private void OnTitleActivated(object o, EventArgs args)
+        private void OnTitleActivated (object o, EventArgs args)
         {
-            if(EnterNextTitle.Sensitive) {
-                OnEnterNextTitleClicked(o, args);
+            if (EnterNextTitle.Sensitive) {
+                OnEnterNextTitleClicked (o, args);
             }
         }
         
         private string last_path = null;
         
-        private void OnCoverButtonClicked(object o, EventArgs args)
+        private void OnCoverButtonClicked (object o, EventArgs args)
         {
-            Banshee.Gui.Dialogs.ImageFileChooserDialog chooser = new Banshee.Gui.Dialogs.ImageFileChooserDialog();
+            Banshee.Gui.Dialogs.ImageFileChooserDialog chooser = new Banshee.Gui.Dialogs.ImageFileChooserDialog ();
             chooser.LocalOnly = true;
             
             try {
                 string path = (TrackSet[currentIndex] as EditorTrack).CoverArtFilename;
-                path = System.IO.Path.GetDirectoryName(path);
+                path = System.IO.Path.GetDirectoryName (path);
                 
-                if(path != null && path != String.Empty) {
-                    chooser.SetCurrentFolder(path);
-                } else if(last_path != null && last_path != String.Empty) {
-                    path = System.IO.Path.GetDirectoryName(last_path);
-                    chooser.SetCurrentFolder(path);
+                if (path != null && path != String.Empty) {
+                    chooser.SetCurrentFolder (path);
+                } else if (last_path != null && last_path != String.Empty) {
+                    path = System.IO.Path.GetDirectoryName (last_path);
+                    chooser.SetCurrentFolder (path);
                 }
             } catch {
             }
             
-            if(chooser.Run() == (int)ResponseType.Ok) {
+            if (chooser.Run () == (int)ResponseType.Ok) {
                 last_path = chooser.Filename;
                 
                 try {
-                    Gdk.Pixbuf pixbuf = new Gdk.Pixbuf(chooser.Filename).ScaleSimple(100, 100, Gdk.InterpType.Bilinear);
-                    SetCoverImage(pixbuf);
+                    Gdk.Pixbuf pixbuf = new Gdk.Pixbuf (chooser.Filename).ScaleSimple (100, 100, Gdk.InterpType.Bilinear);
+                    SetCoverImage (pixbuf);
                     (TrackSet[currentIndex] as EditorTrack).CoverArtFilename = chooser.Filename;
                     (TrackSet[currentIndex] as EditorTrack).CoverArtThumbnail = pixbuf;
                 } catch {
-                    SetCoverImage(null);
+                    SetCoverImage (null);
                 }
             }
             
-            chooser.Destroy();
+            chooser.Destroy ();
         }
         
-        private EditorTrack UpdateCurrent()
+        private EditorTrack UpdateCurrent ()
         {
-            if(currentIndex < 0 || currentIndex >= TrackSet.Count) {
+            if (currentIndex < 0 || currentIndex >= TrackSet.Count) {
                 return null;
             }
                 
@@ -573,7 +580,7 @@ namespace Banshee.Gui.Dialogs
             track.Rating = rating_entry.Value;
             
             try {
-                track.Year = Convert.ToInt32(Year.Text);
+                track.Year = Convert.ToInt32 (Year.Text);
             } catch {
                 track.Year = 0;
             }
@@ -581,42 +588,58 @@ namespace Banshee.Gui.Dialogs
             return track;
         }
 
-        private void OnCancelButtonClicked(object o, EventArgs args)
+        private void OnCancelButtonClicked (object o, EventArgs args)
         {
-            Window.Destroy();
+            Window.Destroy ();
         }
         
-        private void OnSaveButtonClicked(object o, EventArgs args)
+        private List<int> primary_sources = new List<int> ();
+        private void OnSaveButtonClicked (object o, EventArgs args)
         {
-            UpdateCurrent();
+            UpdateCurrent ();
             
-            // TODO improve performance by not notifying each track's source until
-            // we've saved all tracks.  Right now, if you edit two tracks in your library, the
-            // Library is reloaded twice.
-            foreach(EditorTrack track in TrackSet) {
-                SaveTrack(track, true);
+            // TODO wrap in db transaction
+            try {
+                DatabaseTrackInfo.NotifySaved = false;
+
+                primary_sources.Clear ();
+                foreach (EditorTrack track in TrackSet) {
+                    SaveTrack (track);
+
+                    if (track.Track is DatabaseTrackInfo) {
+                        int id = (track.Track as DatabaseTrackInfo).PrimarySourceId;
+                        if (!primary_sources.Contains (id)) {
+                            primary_sources.Add (id);
+                        }
+                    }
+                }
+                
+                EventHandler handler = Saved;
+                if (handler != null) {
+                    handler (this, new EventArgs ());
+                }
+
+                // Finally, notify the affected primary sources
+                foreach (int id in primary_sources) {
+                    PrimarySource.GetById (id).NotifyTracksChanged ();
+                }
+            } finally {
+                DatabaseTrackInfo.NotifySaved = true;
             }
             
-            EventHandler handler = Saved;
-            if(handler != null) {
-                handler(this, new EventArgs());
-            }
-            
-            Window.Destroy();
+            Window.Destroy ();
         }
         
-        private void SaveTrack (EditorTrack track, bool writeToDatabase)
+        private void SaveTrack (EditorTrack track)
         {
             track.Save ();
             
-            if (writeToDatabase) {
-                track.Track.Save ();
+            track.Track.Save ();
                 
-                if (LibrarySchema.WriteMetadata.Get ()) {
-                    SaveToFile (track);
-                }
+            if (LibrarySchema.WriteMetadata.Get ()) {
+                SaveToFile (track);
             }
-                
+
             if (track.Track == ServiceManager.PlayerEngine.CurrentTrack) {
                 ServiceManager.PlayerEngine.TrackInfoUpdated ();
             }
