@@ -35,6 +35,7 @@ using Banshee.ServiceStack;
 using Banshee.MediaEngine;
 using Banshee.Collection;
 
+using Banshee.Gui;
 using Banshee.Sources.Gui;
 
 namespace Banshee.NowPlaying
@@ -42,6 +43,7 @@ namespace Banshee.NowPlaying
     public class NowPlayingSource : Source, IDisposable
     {
         private TrackInfo transitioned_track;
+        private NowPlayingInterface now_playing_interface;
 
         protected override string TypeUniqueId {
             get { return "now-playing"; }
@@ -49,14 +51,40 @@ namespace Banshee.NowPlaying
         
         public NowPlayingSource () : base ("now-playing", Catalog.GetString ("Now Playing"), 0)
         {
+            now_playing_interface = new NowPlayingInterface ();
+        
             Properties.SetString ("Icon.Name", "applications-multimedia");
-            Properties.Set<ISourceContents> ("Nereid.SourceContents", new NowPlayingInterface ());
+            Properties.Set<ISourceContents> ("Nereid.SourceContents", now_playing_interface);
             Properties.Set<bool> ("Nereid.SourceContents.HeaderVisible", false);
+            Properties.SetString ("ActiveSourceUIResource", "ActiveSourceUI.xml");
             
             ServiceManager.SourceManager.AddSource (this);
             
             ServiceManager.PlaybackController.Transition += OnPlaybackControllerTransition;
             ServiceManager.PlaybackController.TrackStarted += OnPlaybackControllerTrackStarted;
+            ServiceManager.PlayerEngine.ConnectEvent (OnTrackInfoUpdated, PlayerEvent.TrackInfoUpdated);
+        }
+        
+        public void Dispose ()
+        {
+            ServiceManager.PlaybackController.Transition -= OnPlaybackControllerTransition;
+            ServiceManager.PlaybackController.TrackStarted -= OnPlaybackControllerTrackStarted;
+            ServiceManager.PlayerEngine.DisconnectEvent (OnTrackInfoUpdated);
+
+            if (now_playing_interface != null) {
+                now_playing_interface.Destroy ();
+                now_playing_interface = null;
+            }
+        }
+        
+        private void OnTrackInfoUpdated (PlayerEventArgs args)
+        {
+            CheckForSwitch ();
+        }
+        
+        private void OnPlaybackControllerTrackStarted (object o, EventArgs args)
+        {
+            CheckForSwitch ();
         }
         
         private void OnPlaybackControllerTransition (object o, EventArgs args)
@@ -64,7 +92,7 @@ namespace Banshee.NowPlaying
             transitioned_track = ServiceManager.PlaybackController.CurrentTrack;
         }
         
-        private void OnPlaybackControllerTrackStarted (object o, EventArgs args)
+        private void CheckForSwitch ()
         {
             TrackInfo current_track = ServiceManager.PlaybackController.CurrentTrack;
             if (current_track != null && transitioned_track != current_track && 
@@ -77,8 +105,18 @@ namespace Banshee.NowPlaying
             }
         }
         
-        public void Dispose ()
+        public override void Activate ()
         {
+            if (now_playing_interface != null) {
+                now_playing_interface.OverrideFullscreen ();
+            }
+        }
+
+        public override void Deactivate ()
+        {
+            if (now_playing_interface != null) {
+                now_playing_interface.RelinquishFullscreen ();
+            }
         }
     }
 }
