@@ -37,18 +37,12 @@ namespace Hyena.Data.Gui
 {
     public class ColumnCellText : ColumnCell
     {
-        private static bool use_cairo_pango;
-        
-        static ColumnCellText ()
-        {
-            use_cairo_pango = String.IsNullOrEmpty (Environment.GetEnvironmentVariable ("USE_GTK_PANGO"));
-            Log.DebugFormat ("Text renderer: {0}", use_cairo_pango ? "Cairo" : "GTK");
-        }
-        
         public delegate string DataHandler ();
     
         private Pango.Weight font_weight = Pango.Weight.Normal;
         private Pango.EllipsizeMode ellipsize_mode = Pango.EllipsizeMode.End;
+        private Pango.Alignment alignment = Pango.Alignment.Left;
+        private double opacity = 1.0;
         private int text_width;
         private int text_height;
         
@@ -58,23 +52,30 @@ namespace Hyena.Data.Gui
     
         public override void Render (CellContext context, StateType state, double cellWidth, double cellHeight)
         {
+            string text = Text;
+            if (String.IsNullOrEmpty (text)) {
+                return;
+            }
+        
             context.Layout.Width = (int)((cellWidth - 8) * Pango.Scale.PangoScale);
             context.Layout.FontDescription.Weight = font_weight;
             context.Layout.Ellipsize = EllipsizeMode;
+            context.Layout.Alignment = alignment;
             
-            context.Layout.SetText (Text);
+            context.Layout.SetText (text);
             context.Layout.GetPixelSize (out text_width, out text_height);
             
-            if (use_cairo_pango) {
-                context.Context.MoveTo (4, ((int)cellHeight - text_height) / 2);
-                context.Context.Color = context.Theme.Colors.GetWidgetColor (
-                    context.TextAsForeground ? GtkColorClass.Foreground : GtkColorClass.Text, state);
-                PangoCairoHelper.ShowLayout (context.Context, context.Layout);
+            context.Context.MoveTo (4, ((int)cellHeight - text_height) / 2);
+            Cairo.Color color = context.Theme.Colors.GetWidgetColor (
+                context.TextAsForeground ? GtkColorClass.Foreground : GtkColorClass.Text, state);
+            if (!context.Sensitive) {
+                color.A = 0.3;
             } else {
-                Style.PaintLayout (context.Widget.Style, context.Drawable, state, !context.TextAsForeground, 
-                    context.Clip, context.Widget, "text", context.Area.X + 4, 
-                    context.Area.Y + (((int)cellHeight - text_height) / 2), context.Layout);
+                color.A = opacity;
             }
+            
+            context.Context.Color = color;
+            PangoCairoHelper.ShowLayout (context.Context, context.Layout);
         }
         
         protected virtual string Text {
@@ -89,6 +90,11 @@ namespace Hyena.Data.Gui
             get { return text_height; }
         }
         
+        protected Pango.Alignment Alignment {
+            get { return alignment; }
+            set { alignment = value; }
+        }
+        
         public virtual Pango.Weight FontWeight {
             get { return font_weight; }
             set { font_weight = value; }
@@ -99,6 +105,11 @@ namespace Hyena.Data.Gui
             set { ellipsize_mode = value; }
         }
         
+        public virtual double Opacity {
+            get { return opacity; }
+            set { opacity = value; }
+        }
+        
         internal static int ComputeRowHeight (Widget widget)
         {
             int w_width, row_height;
