@@ -37,8 +37,6 @@ namespace Hyena.Data.Gui
 {
     public partial class ListView<T> : Widget
     {
-        private int focused_row_index = -1;
-
         private Adjustment vadjustment;
         public Adjustment Vadjustment {
             get { return vadjustment; }
@@ -66,7 +64,7 @@ namespace Hyena.Data.Gui
         {
             int row_limit;
             if (relative_row < 0) {
-                if (focused_row_index == -1) {
+                if (Selection.FocusedIndex == -1) {
                     return false;
                 }
                 
@@ -75,31 +73,33 @@ namespace Hyena.Data.Gui
                 row_limit = Model.Count - 1;
             }
 
-            if (focused_row_index == row_limit) {
+            if (Selection.FocusedIndex == row_limit) {
                 return true;
             }
             
-            int row_index = Math.Min (Model.Count - 1, Math.Max (0, focused_row_index + relative_row));
+            int row_index = Math.Min (Model.Count - 1, Math.Max (0, Selection.FocusedIndex + relative_row));
 
-            if ((modifier & Gdk.ModifierType.ControlMask) != 0) {
-                // Don't change the selection
-            } else if ((modifier & Gdk.ModifierType.ShiftMask) != 0) {
-                // Behave like nautilus: if and arrow key + shift is pressed and the currently focused item
-                // is not selected, select it and don't move the focus or vadjustment.
-                // Otherwise, select the new row and scroll etc as necessary.
-                if ((relative_row * relative_row != 1)) {
-                    Selection.SelectFromFirst (row_index, true);
-                } else if (Selection.Contains (focused_row_index)) {
-                    Selection.SelectFromFirst (row_index, true);
+            if (Selection != null) {
+                if ((modifier & Gdk.ModifierType.ControlMask) != 0) {
+                    // Don't change the selection
+                } else if ((modifier & Gdk.ModifierType.ShiftMask) != 0) {
+                    // Behave like nautilus: if and arrow key + shift is pressed and the currently focused item
+                    // is not selected, select it and don't move the focus or vadjustment.
+                    // Otherwise, select the new row and scroll etc as necessary.
+                    if (relative_row * relative_row != 1) {
+                        Selection.SelectFromFirst (row_index, true);
+                    } else if (Selection.Contains (Selection.FocusedIndex)) {
+                        Selection.SelectFromFirst (row_index, true);
+                    } else {
+                        Selection.Select (Selection.FocusedIndex);
+                        return true;
+                    }
                 } else {
-                    Selection.Select (focused_row_index);
-                    return true;
+                    Selection.Clear (false);
+                    Selection.Select (row_index);
                 }
-            } else {
-                Selection.Clear (false);
-                Selection.Select (row_index);
             }
-
+            
             // Scroll if needed
             double y_at_row = GetYAtRow (row_index);
             if (align_y) {
@@ -109,10 +109,10 @@ namespace Hyena.Data.Gui
                     ScrollTo (y_at_row + RowHeight - (vadjustment.PageSize));
                 }
             } else {
-                ScrollTo (vadjustment.Value + ((row_index - focused_row_index) * RowHeight));
+                ScrollTo (vadjustment.Value + ((row_index - Selection.FocusedIndex) * RowHeight));
             }
 
-            focused_row_index = row_index;
+            Selection.FocusedIndex = row_index;
             InvalidateList ();
             return true;
         }
@@ -174,17 +174,17 @@ namespace Hyena.Data.Gui
 
                 case Gdk.Key.Return:
                 case Gdk.Key.KP_Enter:
-                    if (focused_row_index != -1) {
+                    if (Selection != null && Selection.FocusedIndex != -1) {
                         Selection.Clear (false);
-                        Selection.Select (focused_row_index);
+                        Selection.Select (Selection.FocusedIndex);
                         OnRowActivated ();
                         handled = true;
                     }
                     break;
                 
                 case Gdk.Key.space:
-                    if (focused_row_index != 1) {
-                        Selection.ToggleSelect (focused_row_index);
+                    if (Selection != null && Selection.FocusedIndex != 1) {
+                        Selection.ToggleSelect (Selection.FocusedIndex);
                         handled = true;
                     }
                     break;
@@ -197,12 +197,107 @@ namespace Hyena.Data.Gui
             return base.OnKeyPressEvent (press);
         }
         
+#region Cell Event Proxy        
+        
+        private IInteractiveCell last_icell;
+        private Gdk.Rectangle last_icell_area = Gdk.Rectangle.Zero;
+        
+        private void ProxyEventToCell (Gdk.Event evnt, bool press)
+        {
+            IInteractiveCell icell;
+            Gdk.Rectangle icell_area;
+            bool redraw = ProxyEventToCell (evnt, press, out icell, out icell_area);
+            
+            int xoffset = (int)hadjustment.Value;
+            int yoffset = (int)vadjustment.Value;
+            
+            if (last_icell_area != icell_area) {
+                if (last_icell != null && last_icell.PointerLeaveEvent ()) {
+                    QueueDrawArea (last_icell_area.X - xoffset, last_icell_area.Y - yoffset, 
+                        last_icell_area.Width, last_icell_area.Height);
+                }
+                last_icell = icell;
+                last_icell_area = icell_area;
+            }
+            
+            if (redraw) {
+                QueueDrawArea (icell_area.X - xoffset, icell_area.Y - yoffset, 
+                    icell_area.Width, icell_area.Height);
+            }
+        }
+        
+        private bool ProxyEventToCell (Gdk.Event evnt, bool press, out IInteractiveCell icell, out Gdk.Rectangle icell_area)
+        {
+            icell = null;
+            icell_area = Gdk.Rectangle.Zero;
+            
+            int evnt_x;
+            int evnt_y;
+            
+            Gdk.EventButton evnt_button = evnt as Gdk.EventButton;
+            Gdk.EventMotion evnt_motion = evnt as Gdk.EventMotion;
+            
+            if (evnt_motion != null) {
+                evnt_x = (int)evnt_motion.X;
+                evnt_y = (int)evnt_motion.Y;
+            } else if (evnt_button != null) {
+                evnt_x = (int)evnt_button.X;
+                evnt_y = (int)evnt_button.Y;
+            } else {
+                // Possibly EventCrossing, for the leave event
+                icell = last_icell;
+                return false;
+            }
+            
+            int y = evnt_y - list_interaction_alloc.Y;
+            int x = evnt_x - list_interaction_alloc.X;
+            
+            int row_index = GetRowAtY (y);
+            if (row_index < 0 || row_index >= Model.Count) {
+                return false;
+            }
+            
+            Column column = GetColumnAt (x);
+            if (column == null) {
+                return false;
+            }
+            
+            CachedColumn cached_column = GetCachedColumnForColumn (column);
+            
+            ColumnCell cell = column.GetCell (0);
+            icell = cell as IInteractiveCell;
+            
+            if (icell == null) {
+                return false;
+            }
+            
+            // Turn the view-absolute coordinates into cell-relative coordinates
+            x -= cached_column.X1;
+            int page_offset = (int)vadjustment.Value % RowHeight;
+            y = (y + page_offset) % RowHeight;
+            
+            // Bind the row to the cell and then send it a synthesized input event
+            cell.BindListItem (model[row_index]);
+            
+            bool redraw = (evnt_motion != null)
+                ? icell.MotionEvent (x, y, evnt_motion) : icell.ButtonEvent (x, y, press, evnt_button);
+            
+            icell_area.X = cached_column.X1 + Allocation.X;
+            icell_area.Y = (int)GetYAtRow (row_index) + list_interaction_alloc.Y + Allocation.Y;
+            icell_area.Width = cached_column.Width;
+            icell_area.Height = RowHeight;
+            
+            return redraw;
+        }   
+        
+#endregion
+        
 #region OnButtonPress
 
         protected override bool OnButtonPressEvent (Gdk.EventButton evnt)
         {
             HasFocus = true;
-            if (header_interaction_alloc.Contains ((int)evnt.X, (int)evnt.Y)) {
+            if (header_visible && header_interaction_alloc.Contains ((int)evnt.X, (int)evnt.Y)) {
                 return OnHeaderButtonPressEvent (evnt);
             } else if (list_interaction_alloc.Contains ((int)evnt.X, (int)evnt.Y) && model != null) {
                 return OnListButtonPressEvent (evnt);
@@ -252,11 +347,13 @@ namespace Hyena.Data.Gui
             
             int row_index = GetRowAtY (y);
 
-            if (row_index >= Model.Count) {
+            if (row_index < 0 || row_index >= Model.Count) {
                 return true;
             }
             
-            if (evnt.Button == 1 && evnt.Type != Gdk.EventType.TwoButtonPress && 
+            ProxyEventToCell (evnt, true);
+            
+            if (Selection != null && evnt.Button == 1 && evnt.Type != Gdk.EventType.TwoButtonPress && 
                 (evnt.State & Gdk.ModifierType.ControlMask) == 0 && Selection.Contains (row_index)) {
                 return true;
             }
@@ -268,7 +365,7 @@ namespace Hyena.Data.Gui
 
             if (evnt.Button == 1 && evnt.Type == Gdk.EventType.TwoButtonPress) {
                 OnRowActivated ();
-            } else {
+            } else if (Selection != null) {
                 if ((evnt.State & Gdk.ModifierType.ControlMask) != 0) {
                     if (evnt.Button == 3) {
                         if (!Selection.Contains (row_index)) {
@@ -322,15 +419,15 @@ namespace Hyena.Data.Gui
                 pressed_column_is_dragging = false;
                 pressed_column_index = -1;
                 GdkWindow.Cursor = null;
-                InvalidateHeader ();
-                InvalidateList ();
+                QueueDraw ();
                 return true;
             }
             
-            if (header_interaction_alloc.Contains ((int)evnt.X, (int)evnt.Y)) {
+            if (header_visible && header_interaction_alloc.Contains ((int)evnt.X, (int)evnt.Y)) {
                 return OnHeaderButtonRelease (evnt);
             } else if (list_interaction_alloc.Contains ((int)evnt.X, (int)evnt.Y) && model != null &&
                 (evnt.State & (Gdk.ModifierType.ShiftMask | Gdk.ModifierType.ControlMask)) == 0) {
+                ProxyEventToCell (evnt, false);
                 return OnListButtonRelease (evnt);
             }
 
@@ -339,17 +436,21 @@ namespace Hyena.Data.Gui
         
         private bool OnHeaderButtonRelease (Gdk.EventButton evnt)
         {
-            Column column = column_cache[pressed_column_index].Column;
-            if (column != null && Model is ISortable && column is ISortableColumn) {
-                ((ISortable)Model).Sort ((ISortableColumn)column);
-                Model.Reload ();
-                RecalculateColumnSizes ();
-                RegenerateColumnCache ();
-                InvalidateHeader ();
+            if (pressed_column_index >= 0 && pressed_column_index < column_cache.Length) {
+                Column column = column_cache[pressed_column_index].Column;
+                if (column != null && Model is ISortable && column is ISortableColumn) {
+                    ((ISortable)Model).Sort ((ISortableColumn)column);
+                    Model.Reload ();
+                    RecalculateColumnSizes ();
+                    RegenerateColumnCache ();
+                    InvalidateHeader ();
+                }
+                
+                pressed_column_index = -1;
+                return true;
+            } else {
+                return false;
             }
-            
-            pressed_column_index = -1;
-            return true;
         }
         
         private bool OnListButtonRelease (Gdk.EventButton evnt)
@@ -369,11 +470,12 @@ namespace Hyena.Data.Gui
                 return true;
             }
             
-            if (Selection.Contains (row_index) && Selection.Count > 1) {
+            if (Selection != null && Selection.Contains (row_index) && Selection.Count > 1) {
                 Selection.Clear (false);
                 Selection.Select (row_index);
                 FocusRow (row_index);
             }
+            
             return true;
         }
         
@@ -405,6 +507,8 @@ namespace Hyena.Data.Gui
                 ResizeColumn (x);
             }
             
+            ProxyEventToCell (evnt, false);
+            
             return true;
         }
         
@@ -458,9 +562,11 @@ namespace Hyena.Data.Gui
         protected override bool OnLeaveNotifyEvent (Gdk.EventCrossing evnt)
         {
             GdkWindow.Cursor = null;
+            if (evnt.Mode == Gdk.CrossingMode.Normal) {
+                ProxyEventToCell (evnt, false);
+            }
             return base.OnLeaveNotifyEvent (evnt);
         }
-
         
         protected override bool OnFocusInEvent (Gdk.EventFocus evnt)
         {
@@ -474,16 +580,20 @@ namespace Hyena.Data.Gui
         
         protected virtual void OnRowActivated ()
         {
-            if (focused_row_index != -1) {
+            if (Selection.FocusedIndex != -1) {
                 RowActivatedHandler<T> handler = RowActivated;
                 if (handler != null) {
-                    handler (this, new RowActivatedArgs<T> (focused_row_index, model[focused_row_index]));
+                    handler (this, new RowActivatedArgs<T> (Selection.FocusedIndex, model[Selection.FocusedIndex]));
                 }
             }
         }
         
         protected int GetRowAtY (int y)
         {
+            if (y < 0) {
+                return -1;
+            }
+            
             int page_offset = (int)vadjustment.Value % RowHeight;
             int first_row = (int)vadjustment.Value / RowHeight;
             int row_offset = (y + page_offset) / RowHeight;
@@ -493,13 +603,13 @@ namespace Hyena.Data.Gui
 
         protected double GetYAtRow (int row)
         {
-            double y = (double) RowHeight * row;
+            double y = (double)RowHeight * row;
             return y;
         }
           
         private void FocusRow (int index)
         {
-            focused_row_index = index;
+            Selection.FocusedIndex = index;
         }
 
 #endregion
@@ -537,19 +647,24 @@ namespace Hyena.Data.Gui
                 }
             }
             
-            hadjustment.Change ();
-            vadjustment.Change ();
+            if (hadjustment != null) {
+                hadjustment.Change ();
+            }
+
+            if (vadjustment != null) {
+                vadjustment.Change ();
+            }
         }
         
         private void OnHadjustmentChanged (object o, EventArgs args)
         {
             InvalidateHeader ();
-            InvalidateList (false);
+            InvalidateList ();
         }
         
         private void OnVadjustmentChanged (object o, EventArgs args)
         {
-            InvalidateList (false);
+            InvalidateList ();
         }
         
         public void ScrollTo (double val)
@@ -559,7 +674,9 @@ namespace Hyena.Data.Gui
         
         private void ScrollTo (Adjustment adjustment, double val)
         {
-            adjustment.Value = Math.Max (0.0, Math.Min (val, adjustment.Upper - adjustment.PageSize));
+            if (adjustment != null) {
+                adjustment.Value = Math.Max (0.0, Math.Min (val, adjustment.Upper - adjustment.PageSize));
+            }
         }
 
         public void ScrollTo (int index)
