@@ -33,14 +33,14 @@ using Gtk;
 
 namespace Hyena.Data.Gui
 {
-    public partial class ListView<T> : Container
+    public partial class ListView<T> : Widget
     {
         public void SetModel (IListModel<T> model)
         {
             SetModel (model, 0.0);
         }
 
-        public void SetModel (IListModel<T> value, double vpos)
+        public virtual void SetModel (IListModel<T> value, double vpos)
         {
             if (model == value) {
                 return;
@@ -69,7 +69,7 @@ namespace Hyena.Data.Gui
 
         private void RefreshViewForModel (double? vpos)
         {
-            UpdateAdjustments (null, null);
+            UpdateAdjustments ();
 
             if (vpos != null) {
                 ScrollTo ((double) vpos);
@@ -87,19 +87,18 @@ namespace Hyena.Data.Gui
                     }
 
                     if (!selection_in_view) {
-                        ScrollToRow (Selection.Ranges[0].Start + 1 - RowsInView/2);
+                        CenterOn (Selection.Ranges[0].Start);
                     }
                 } else {
                     ScrollTo (vadjustment.Value);
                 }
             }
             
-            if (Model != null) {
+            if (Model != null && Selection != null) {
                 Selection.MaxIndex = Model.Count - 1;
             }
             
             if (Parent is ScrolledWindow) {
-                render_everything = true;
                 Parent.QueueDraw ();
             }
         }
