@@ -39,78 +39,84 @@ namespace Hyena.Widgets
     public abstract class AnimatedBox : Container
     {
         private readonly Stage<AnimatedWidget> stage = new Stage<AnimatedWidget> ();
-        private readonly Queue<AnimatedWidget> expired = new Queue<AnimatedWidget> ();
         private readonly LinkedList<AnimatedWidget> children = new LinkedList<AnimatedWidget> ();
-        private readonly object children_mutex = new object ();
-        
-        private int spacing;
-        
-        private int start_spacing;
-        protected int StartSpacing {
-            get { return start_spacing; }
-        }
-        
-        private int end_spacing;
-        protected int EndSpacing {
-            get { return end_spacing; }
-        }
+        private readonly SingleActorStage border_stage = new Hyena.Gui.Theatrics.SingleActorStage ();
+        private readonly bool horizontal;
         
         private uint duration = 500;
         private Easing easing = Easing.Linear;
         private Blocking blocking = Blocking.Upstage;
+        private int start_padding;
+        private int end_padding;
+        private int spacing;
+        private int start_spacing;
+        private int end_spacing;
+        private int active_count;
         
-        protected AnimatedBox ()
+        private int start_border;
+        private int end_border;
+        private double border_bias;
+        private Easing border_easing;
+        private AnimationState border_state;
+        
+        protected AnimatedBox (bool horizontal)
         {
+            WidgetFlags |= WidgetFlags.NoWindow;
+            this.horizontal = horizontal;
             stage.ActorStep += OnActorStep;
-            stage.Iteration += OnIteration;
+            border_stage.Iteration += OnBorderIteration;
         }
         
-#region Private Methods
+#region Private
+        
+        private double Percent {
+            get { return border_stage.Actor.Percent * border_bias + (1.0 - border_bias); }
+        }
         
         private bool OnActorStep (Actor<AnimatedWidget> actor)
         {
-            lock (actor.Target) {
-                switch (actor.Target.AnimationState) {
-                case AnimationState.Coming:
-                    actor.Target.Percent = actor.Percent;
-                    if (actor.Expired) {
-                        actor.Target.AnimationState = AnimationState.Idle;
-                        return false;
-                    }
-                    break;
-                case AnimationState.IntendingToGo:
-                    actor.Target.AnimationState = AnimationState.Going;
-                    actor.Target.Bias = actor.Percent;
-                    actor.Reset ((uint)(actor.Target.Duration * actor.Percent));
-                    break;
-                case AnimationState.Going:
-                    if (actor.Expired) {
-                        lock (children_mutex) {
-                            children.Remove (actor.Target.Node);
-                        }
-                        expired.Enqueue (actor.Target);
-                        return false;
-                    } else {
-                        actor.Target.Percent = 1.0 - actor.Percent;
-                    }
-                    break;
+            switch (actor.Target.AnimationState) {
+            case AnimationState.Coming:
+                actor.Target.Percent = actor.Percent;
+                if (actor.Expired) {
+                    actor.Target.AnimationState = AnimationState.Idle;
+                    return false;
                 }
+                break;
+            case AnimationState.IntendingToGo:
+                actor.Target.AnimationState = AnimationState.Going;
+                actor.Target.Bias = actor.Percent;
+                actor.Reset ((uint)(actor.Target.Duration * actor.Percent));
+                break;
+            case AnimationState.Going:
+                if (actor.Expired) {
+                    actor.Target.Unparent ();
+                    children.Remove (actor.Target.Node);
+                    return false;
+                } else {
+                    actor.Target.Percent = 1.0 - actor.Percent;
+                }
+                break;
             }
             
             return true;
         }
         
-        private void OnIteration (object sender, EventArgs args)
+        private void OnBorderIteration (object sender, EventArgs args)
         {
-            // When widgets are disposed, their hash code changes (zee uber lame).
-            // We would otherwise do this up in OnActorStep, but the has code needs
-            // to remain the same so that the actor can be removed from the stage.
-            while (expired.Count > 0) {
-                AnimatedWidget widget = expired.Dequeue ();
-                widget.Unparent ();
-                widget.Dispose ();
+            if (border_stage.Actor == null) {
+                if (border_state == AnimationState.Coming) {
+                    start_border = start_padding;
+                    end_border = end_padding;
+                } else {
+                    start_border = end_border = 0;
+                }
+                border_state = AnimationState.Idle;
+            } else {
+                double percent = border_state == AnimationState.Coming ? Percent : 1.0 - Percent;
+                start_border = Choreographer.Compose (percent, start_padding, border_easing);
+                end_border = Choreographer.Compose (percent, end_padding, border_easing);
             }
-            
             QueueResizeNoRedraw ();
         }
         
@@ -119,6 +125,169 @@ namespace Hyena.Widgets
             RemoveCore ((AnimatedWidget)sender);
         }
         
+        private void RecalculateSpacings ()
+        {
+            int skip_count = 0;
+            
+            foreach (AnimatedWidget animated_widget in Widgets) {
+                animated_widget.QueueResizeNoRedraw ();
+                if (skip_count > 1) {
+                    skip_count--;
+                    continue;
+                }
+                AnimatedWidget widget = animated_widget;
+                
+                if (skip_count == 0) {
+                    widget.StartPadding = start_spacing;
+                } else {
+                    skip_count--;
+                }
+                widget.EndPadding = end_spacing;
+                
+                if (widget.Node.Previous == null) {
+                    while (true) {
+                        widget.StartPadding = 0;
+                        if (widget.AnimationState == AnimationState.Coming ||
+                            widget.AnimationState == AnimationState.Idle || widget.Node.Next == null) {
+                            break;
+                        }
+                        widget.EndPadding = spacing;
+                        widget = widget.Node.Next.Value;
+                        skip_count++;
+                    }
+                }
+                
+                if (widget.Node.Next == null) {
+                    while (true) {
+                        widget.EndPadding = 0;
+                        if (widget.AnimationState == AnimationState.Coming ||
+                            widget.AnimationState == AnimationState.Idle || widget.Node.Previous == null) {
+                            break;
+                        }
+                        widget.StartPadding = spacing;
+                        widget = widget.Node.Previous.Value;
+                    }
+                }
+            }
+        }
+        
+#endregion
+        
+#region Protected Overrides
+        
+        protected override void OnAdded (Widget widget)
+        {
+            PackStart (widget, duration, easing, blocking);
+        }
+        
+        protected override void OnSizeRequested (ref Requisition requisition)
+        {
+            int width = 0;
+            int height = 0;
+            
+            if (horizontal) {
+                width = start_border + end_border;
+            } else {
+                height = start_border + end_border;
+            }
+            
+            foreach (AnimatedWidget widget in Widgets) {
+                Requisition req = widget.SizeRequest ();
+                if (horizontal) {
+                    width += req.Width;
+                    height = Math.Max (height, req.Height);
+                } else {
+                    width = Math.Max (width, req.Width);
+                    height += req.Height;
+                }
+            }
+            
+            requisition.Width = width;
+            requisition.Height = height;
+        }
+
+        protected override void OnSizeAllocated (Rectangle allocation)
+        {
+            base.OnSizeAllocated (allocation);
+            if (horizontal) {
+                allocation.X += start_border;
+                allocation.Y += (int)BorderWidth;
+                allocation.Height -= (int)BorderWidth * 2;
+            } else {
+                allocation.X += (int)BorderWidth;
+                allocation.Y += start_border;
+                allocation.Width -= (int)BorderWidth * 2;
+            }
+            
+            foreach (AnimatedWidget widget in Widgets) {
+                if (horizontal) {
+                    allocation.Width = widget.Width;
+                    widget.SizeAllocate (allocation);
+                    allocation.X += allocation.Width;
+                } else {
+                    allocation.Height = widget.Height;
+                    widget.SizeAllocate (allocation);
+                    allocation.Y += allocation.Height;
+                }
+            }
+        }
+        
+        protected override void ForAll (bool include_internals, Callback callback)
+        {
+            foreach (AnimatedWidget child in Widgets) {
+                callback (child);
+            }
+        }
+        
+#endregion
+        
+#region Public
+                
+#region Properties
+        
+        public uint Duration {
+            get { return duration; }
+            set { duration = value; }
+        }
+        
+        public Easing Easing {
+            get { return easing; }
+            set { easing = value; }
+        }
+        
+        public Blocking Blocking {
+            get { return blocking; }
+            set { blocking = value; }
+        }
+        
+        public int Spacing {
+            get { return spacing; }
+            set {
+                spacing = value;
+                double half = (double)value / 2.0;
+                start_spacing = (int)Math.Ceiling (half);
+                end_spacing = (int)Math.Floor (half);
+            }
+        }
+        
+        public int StartPadding {
+            get { return start_padding - (int)BorderWidth; }
+            set { start_padding = value + (int)BorderWidth; }
+        }
+        
+        public int EndPadding {
+            get { return end_padding - (int)BorderWidth; }
+            set { end_padding = value + (int)BorderWidth; }
+        }
+        
+        internal IEnumerable<AnimatedWidget> Widgets {
+            get {
+                foreach (AnimatedWidget child in children) {
+                    yield return child;
+                }
+            }
+        }
+        
 #endregion
     
 #region Pack Methods
@@ -160,10 +329,7 @@ namespace Hyena.Widgets
         
         public void PackStart (Widget widget, uint duration, Easing easing, Blocking blocking)
         {
-            AnimatedWidget animated_widget = Pack (widget, duration, easing, blocking);
-            lock (children_mutex) {
-                animated_widget.Node = children.AddFirst (animated_widget);
-            }
+            Pack (widget, duration, easing, blocking, false);
         }
         
         public void PackEnd (Widget widget)
@@ -203,23 +369,35 @@ namespace Hyena.Widgets
         
         public void PackEnd (Widget widget, uint duration, Easing easing, Blocking blocking)
         {
-            AnimatedWidget animated_widget = Pack (widget, duration, easing, blocking);
-            lock (children_mutex) {
-                animated_widget.Node = children.AddLast (animated_widget);
-            }
+            Pack (widget, duration, easing, blocking, true);
         }
         
-        private AnimatedWidget Pack (Widget widget, uint duration, Easing easing, Blocking blocking)
+        private void Pack (Widget widget, uint duration, Easing easing, Blocking blocking, bool end)
         {
             if (widget == null) {
                 throw new ArgumentNullException ("widget");
             }
             
-            AnimatedWidget animated_widget = new AnimatedWidget (widget, duration, easing, blocking);
+            AnimatedWidget animated_widget = new AnimatedWidget (widget, duration, easing, blocking, horizontal);
             animated_widget.Parent = this;
             animated_widget.WidgetDestroyed += OnWidgetDestroyed;
             stage.Add (animated_widget, duration);
-            return animated_widget;
+            animated_widget.Node = end 
+                ? children.AddLast (animated_widget)
+                : children.AddFirst (animated_widget);
+            
+            RecalculateSpacings ();
+            if (active_count == 0) {
+                if (border_state == AnimationState.Going) {
+                    border_bias = Percent;
+                } else {
+                    border_easing = easing;
+                    border_bias = 1.0;
+                }
+                border_state = AnimationState.Coming;
+                border_stage.Reset ((uint)(duration * border_bias));
+            }
+            active_count++;
         }
         
 #endregion
@@ -228,45 +406,45 @@ namespace Hyena.Widgets
         
         public new void Remove (Widget widget)
         {
-            RemoveCore (widget, 0, Easing.None, Blocking.None);
+            RemoveCore (widget, 0, 0, 0, false, false);
         }
         
         public void Remove (Widget widget, uint duration)
         {
-            RemoveCore (widget, duration, Easing.None, Blocking.None);
+            RemoveCore (widget, duration, 0, 0, false, false);
         }
         
         public void Remove (Widget widget, Easing easing)
         {
-            RemoveCore (widget, 0, easing, Blocking.None);
+            RemoveCore (widget, 0, easing, 0, true, false);
         }
         
         public void Remove (Widget widget, uint duration, Easing easing)
         {
-            RemoveCore (widget, duration, easing, Blocking.None);
+            RemoveCore (widget, duration, easing, 0, true, false);
         }
         
         public void Remove (Widget widget, Blocking blocking)
         {
-            RemoveCore (widget, 0, Easing.None, blocking);
+            RemoveCore (widget, 0, 0, blocking, false, true);
         }
         
         public void Remove (Widget widget, uint duration, Blocking blocking)
         {
-            RemoveCore (widget, duration, Easing.None, blocking);
+            RemoveCore (widget, duration, 0, blocking, false, true);
         }
         
         public void Remove (Widget widget, Easing easing, Blocking blocking)
         {
-            RemoveCore (widget, 0, easing, blocking);
+            RemoveCore (widget, 0, easing, blocking, true, true);
         }
         
         public void Remove (Widget widget, uint duration, Easing easing, Blocking blocking)
         {
-            RemoveCore (widget, duration, easing, blocking);
+            RemoveCore (widget, duration, easing, blocking, true, true);
         }
         
-        private void RemoveCore (Widget widget, uint duration, Easing easing, Blocking blocking)
+        private void RemoveCore (Widget widget, uint duration, Easing easing, Blocking blocking, bool use_easing, bool use_blocking)
         {
             if (widget == null) {
                 throw new ArgumentNullException ("widget");
@@ -284,52 +462,61 @@ namespace Hyena.Widgets
                 throw new ArgumentException ("Cannot remove the specified widget because it has not been added to this container or it has already been removed.", "widget");
             }
             
-            RemoveCore (animated_widget, duration, easing, blocking);
+            RemoveCore (animated_widget, duration, easing, blocking, use_easing, use_blocking);
+            RecalculateSpacings ();
         }
         
         private void RemoveCore (AnimatedWidget widget)
         {
-            RemoveCore (widget, widget.Duration, widget.Easing, widget.Blocking);
+            RemoveCore (widget, widget.Duration, 0, 0, false, false);
         }
         
-        private void RemoveCore (AnimatedWidget widget, uint duration, Easing easing, Blocking blocking)
+        private void RemoveCore (AnimatedWidget widget, uint duration, Easing easing, Blocking blocking, bool use_easing, bool use_blocking)
         {
-            lock (widget) {
-                if (duration > 0) {
-                    widget.Duration = duration;
-                }
-                
-                if (easing != Easing.None) {
-                    widget.Easing = easing;
-                }
-                
-                if (blocking != Blocking.None) {
-                    widget.Blocking = blocking;
-                }
+            if (duration > 0) {
+                widget.Duration = duration;
+            }
             
-                if (widget.AnimationState == AnimationState.Coming) {
-                    widget.AnimationState = AnimationState.IntendingToGo;
-                } else {
-                    if (widget.Easing == Easing.QuadraticIn) {
-                        widget.Easing = Easing.QuadraticOut;
-                    } else if (widget.Easing == Easing.QuadraticOut) {
-                        widget.Easing = Easing.QuadraticIn;
-                    } else if (widget.Easing == Easing.ExponentialIn) {
-                        widget.Easing = Easing.ExponentialOut;
-                    } else if (widget.Easing == Easing.ExponentialOut) {
-                        widget.Easing = Easing.ExponentialIn;
-                    }
-                    
-                    widget.AnimationState = AnimationState.Going;
-                    stage.Add (widget, widget.Duration);
+            if (use_easing) {
+                widget.Easing = easing;
+            }
+            
+            if (use_blocking) {
+                widget.Blocking = blocking;
+            }
+        
+            if (widget.AnimationState == AnimationState.Coming) {
+                widget.AnimationState = AnimationState.IntendingToGo;
+            } else {
+                if (widget.Easing == Easing.QuadraticIn) {
+                    widget.Easing = Easing.QuadraticOut;
+                } else if (widget.Easing == Easing.QuadraticOut) {
+                    widget.Easing = Easing.QuadraticIn;
+                } else if (widget.Easing == Easing.ExponentialIn) {
+                    widget.Easing = Easing.ExponentialOut;
+                } else if (widget.Easing == Easing.ExponentialOut) {
+                    widget.Easing = Easing.ExponentialIn;
                 }
+                widget.AnimationState = AnimationState.Going;
+                stage.Add (widget, widget.Duration);
+            }
+            
+            duration = widget.Duration;
+            easing = widget.Easing;
+            
+            active_count--;
+            if (active_count == 0) {
+                if (border_state == AnimationState.Coming) {
+                    border_bias = Percent;
+                } else {
+                    border_easing = easing;
+                    border_bias = 1.0;
+                }
+                border_state = AnimationState.Going;
+                border_stage.Reset ((uint)(duration * border_bias));
             }
         }
         
-#endregion
-        
-#region Other Public Methods
-        
         public void RemoveAll ()
         {
             foreach (AnimatedWidget child in Widgets) {
@@ -337,8 +524,11 @@ namespace Hyena.Widgets
                     RemoveCore (child);
                 }
             }
+            RecalculateSpacings ();
         }
         
+#endregion
+        
         public bool Contains (Widget widget)
         {
             foreach (AnimatedWidget child in Widgets) {
@@ -346,75 +536,9 @@ namespace Hyena.Widgets
                     return true;
                 }
             }
-            
             return false;
         }
         
-#endregion
-        
-#region Overrides
-        
-        protected override void OnAdded (Widget widget)
-        {
-            PackStart (widget, duration, easing, blocking);
-        }
-        
-        protected override void OnRealized ()
-        {
-            WidgetFlags |= WidgetFlags.Realized | WidgetFlags.NoWindow;
-            GdkWindow = Parent.GdkWindow;
-            base.OnRealized ();
-        }
-        
-        protected override void ForAll (bool include_internals, Callback callback)
-        {
-            foreach (AnimatedWidget child in Widgets) {
-                callback (child);
-            }
-        }
-        
-#endregion
-        
-#region Properties
-        
-        public int Spacing {
-            get { return spacing; }
-            set {
-                if (value < 0) {
-                    throw new ArgumentOutOfRangeException ("value", "Spacing cannot be less than 0.");
-                }
-                spacing = value;
-                double half = (double)spacing / 2.0;
-                start_spacing = (int)Math.Floor (half);
-                end_spacing = (int)Math.Ceiling (half);
-            }
-        }
-        
-        public uint Duration {
-            get { return duration; }
-            set { duration = value; }
-        }
-        
-        public Easing Easing {
-            get { return easing; }
-            set { easing = value; }
-        }
-        
-        public Blocking Blocking {
-            get { return blocking; }
-            set { blocking = value; }
-        }
-        
-        internal IEnumerable<AnimatedWidget> Widgets {
-            get {
-                lock (children_mutex) {
-                    foreach (AnimatedWidget child in children) {
-                        yield return child;
-                    }
-                }
-            }
-        }
-        
 #endregion
         
     }
