@@ -34,22 +34,25 @@ using System.Text;
 
 namespace Hyena.Data.Sqlite
 {
-    public abstract class SqliteModelProvider<T>
+    public class SqliteModelProvider<T> where T : new ()
     {
         private readonly List<DatabaseColumn> columns = new List<DatabaseColumn> ();
         private readonly List<DatabaseColumn> select_columns = new List<DatabaseColumn> ();
         private readonly List<VirtualDatabaseColumn> virtual_columns = new List<VirtualDatabaseColumn> ();
         
         private DatabaseColumn key;
+        private int key_select_column_index;
         private HyenaSqliteConnection connection;
         
         private HyenaSqliteCommand create_command;
         private HyenaSqliteCommand insert_command;
         private HyenaSqliteCommand update_command;
+        private HyenaSqliteCommand delete_command;
         private HyenaSqliteCommand select_command;
         private HyenaSqliteCommand select_range_command;
         private HyenaSqliteCommand select_single_command;
         
+        private string table_name;
         private string primary_key;
         private string select;
         private string from;
@@ -57,24 +60,51 @@ namespace Hyena.Data.Sqlite
         
         private const string HYENA_DATABASE_NAME = "hyena_database_master";
 
-        public abstract string TableName { get; }
-        protected abstract int ModelVersion { get; }
-        protected abstract int DatabaseVersion { get; }
-        protected abstract void MigrateTable (int old_version);
-        protected abstract void MigrateDatabase (int old_version);
-        protected abstract T MakeNewObject (int offset);
+        public virtual string TableName { get { return table_name; } }
+
+        protected virtual int ModelVersion { get { return 1; } }
+
+        protected virtual int DatabaseVersion { get { return 1; } }
+
+        protected virtual void MigrateTable (int old_version)
+        {
+        }
+
+        protected virtual void MigrateDatabase (int old_version)
+        {
+        }
+
+        protected virtual T MakeNewObject ()
+        {
+            return new T ();
+        }
         
         protected virtual string HyenaTableName {
             get { return "HyenaModelVersions"; }
         }
         
+        public HyenaSqliteConnection Connection {
+            get { return connection; }
+        }
+        
         protected SqliteModelProvider (HyenaSqliteConnection connection)
         {
             this.connection = connection;
         }
+        
+        public SqliteModelProvider (HyenaSqliteConnection connection, string table_name) : this (connection)
+        {
+            this.table_name = table_name;
+            Init ();
+        }
 
         protected void Init ()
         {
+            foreach (FieldInfo field in typeof(T).GetFields (BindingFlags.Instance | BindingFlags.Public)) {
+                foreach (Attribute attribute in field.GetCustomAttributes (true)) {
+                    AddColumn (field, attribute);
+                }
+            }
             foreach (FieldInfo field in typeof(T).GetFields (BindingFlags.Instance | BindingFlags.NonPublic)) {
                 foreach (Attribute attribute in field.GetCustomAttributes (true)) {
                     AddColumn (field, attribute);
@@ -90,11 +120,12 @@ namespace Hyena.Data.Sqlite
                     AddColumn (property, attribute);
                 }
             }
-            
             if (key == null) {
                 throw new Exception (String.Format ("The {0} table does not have a primary key", TableName));
             }
             
+            key_select_column_index = select_columns.IndexOf (key);
+            
             CheckVersion ();
             CheckTable ();
         }
@@ -154,7 +185,7 @@ namespace Hyena.Data.Sqlite
                 HyenaTableName, name, version));
         }
         
-        private void CheckTable ()
+        protected void CheckTable ()
         {
             //Console.WriteLine ("In {0} checking for table {1}", this, TableName);
             IDictionary<string, string> schema = connection.GetSchema (TableName);
@@ -236,18 +267,32 @@ namespace Hyena.Data.Sqlite
             }
         }
         
-        public void Save (T target)
+        protected void CreateIndex (string name, string columns)
         {
-            if (((int)key.GetValue (target)) > 0) {
-                Update (target);
-            } else {
-                key.SetValue (target, Insert (target));
+            Connection.Execute (String.Format (
+                "CREATE INDEX {0} ON {1} ({2})",
+                name, TableName, columns
+            ));
+        }
+        
+        public virtual void Save (T target)
+        {
+            try {
+                if (Convert.ToInt32 (key.GetRawValue (target)) > 0) {
+                    Update (target);
+                } else {
+                    key.SetValue (target, Insert (target));
+                }
+            } catch (Exception e) {
+                Hyena.Log.Exception (e); 
+                Hyena.Log.DebugFormat ("type of key value: {0}", key.GetRawValue (target).GetType ());
+                throw;
             }
         }
         
-        protected virtual void PrepareInsertCommand (T target)
+        protected virtual object [] GetInsertParams (T target)
         {
-            // TODO create an instance variable object array and reuse it?
+            // TODO create an instance variable object array and reuse it? beware threading issues
             object [] values = new object [columns.Count - 1];
             int j = 0;
             for (int i = 0; i < columns.Count; i++) {
@@ -256,18 +301,17 @@ namespace Hyena.Data.Sqlite
                     j++;
                 }
             }
-            InsertCommand.ApplyValues (values);
+            return values;
         }
         
         protected int Insert (T target)
         {
-            PrepareInsertCommand (target);
-            return connection.Execute (InsertCommand);
+            return connection.Execute (InsertCommand, GetInsertParams (target));
         }
 
-        protected virtual void PrepareUpdateCommand (T target)
+        protected object [] GetUpdateParams (T target)
         {
-            // TODO create an instance variable object array and reuse it?
+            // TODO create an instance variable object array and reuse it? beware threading issues
             object [] values = new object [columns.Count];
             int j = 0;
             for (int i = 0; i < columns.Count; i++) {
@@ -277,18 +321,17 @@ namespace Hyena.Data.Sqlite
                 }
             }
             values[j] = key.GetValue (target);
-            UpdateCommand.ApplyValues (values);
+            return values;
         }
         
         protected void Update (T target)
         {
-            PrepareUpdateCommand (target);
-            connection.Execute (UpdateCommand);
+            connection.Execute (UpdateCommand, GetUpdateParams (target));
         }
         
-        public T Load (IDataReader reader, int index)
+        public virtual T Load (IDataReader reader)
         {
-            T item = MakeNewObject (index);
+            T item = MakeNewObject ();
             Load (reader, item);
             return item;
         }
@@ -316,51 +359,91 @@ namespace Hyena.Data.Sqlite
             }
         }
         
-        protected virtual void PrepareSelectCommand ()
-        {
-        }
-        
         public IEnumerable<T> FetchAll ()
         {
-            PrepareSelectCommand ();
-            int i = 1;
             using (IDataReader reader = connection.Query (SelectCommand)) {
                 while (reader.Read ()) {
-                    yield return Load (reader, i++);
+                    yield return Load (reader);
                 }
             }
         }
-        
-        protected virtual void PrepareSelectRangeCommand (int offset, int limit)
+
+        public T FetchFirstMatching (string condition, params object [] vals)
         {
-            SelectRangeCommand.ApplyValues (offset, limit);
+            foreach (T item in FetchAllMatching (String.Format ("{0} LIMIT 1", condition), vals)) {
+                return item;
+            }
+            return default(T);
+        }
+        
+        public IEnumerable<T> FetchAllMatching (string condition, params object [] vals)
+        {
+            HyenaSqliteCommand fetch_matching_command = new HyenaSqliteCommand (String.Format ("{0} AND {1}", SelectCommand.Text, condition));
+            using (IDataReader reader = connection.Query (fetch_matching_command, vals)) {
+                while (reader.Read ()) {
+                    yield return Load (reader);
+                }
+            }
         }
         
         public IEnumerable<T> FetchRange (int offset, int limit)
         {
-            PrepareSelectRangeCommand (offset, limit);
-            using (IDataReader reader = connection.Query (SelectRangeCommand)) {
+            using (IDataReader reader = connection.Query (SelectRangeCommand, offset, limit)) {
                 while (reader.Read ()) {
-                    yield return Load (reader, offset++);
+                    yield return Load (reader);
                 }
             }
         }
         
-        protected virtual void PrepareSelectSingleCommand (object id)
-        {
-            SelectSingleCommand.ApplyValues (id);
-        }
-        
         public T FetchSingle (int id)
         {
-            PrepareSelectSingleCommand (id);
-            using (IDataReader reader = connection.Query (SelectSingleCommand)) {
+            return FetchSingle ((long) id);
+        }
+        
+        public virtual T FetchSingle (long id)
+        {
+            using (IDataReader reader = connection.Query (SelectSingleCommand, id)) {
                 if (reader.Read ()) {
-                    return Load (reader, id);
+                    return Load (reader);
                 }
             }
             return default(T);
         }
+        
+        protected long PrimaryKeyFor (T item)
+        {
+            return Convert.ToInt64 (key.GetValue (item));
+        }
+        
+        protected long PrimaryKeyFor (IDataReader reader)
+        {
+            return Convert.ToInt64 (reader[key_select_column_index]);
+        }
+        
+        public virtual void Delete (long id)
+        {
+            if (id > 0)
+                connection.Execute (DeleteCommand, id);
+        }
+        
+        public void Delete (T item)
+        {
+            Delete (PrimaryKeyFor (item));
+        }
+        
+        public virtual void Delete (IEnumerable<T> items)
+        {
+            List<long> ids = new List<long> ();
+            long id;
+            foreach (T item in items) {
+                id = PrimaryKeyFor (item);
+                if (id > 0)
+                    ids.Add (id);
+            }
+            
+            if (ids.Count > 0)
+                connection.Execute (DeleteCommand, ids.ToArray ());
+        }
 
         public bool Refresh (T item)
         {
@@ -371,8 +454,7 @@ namespace Hyena.Data.Sqlite
             if (id < 1)
                 return false;
 
-            PrepareSelectSingleCommand (id);
-            using (IDataReader reader = connection.Query (SelectSingleCommand)) {
+            using (IDataReader reader = connection.Query (SelectSingleCommand, id)) {
                 if (reader.Read ()) {
                     Load (reader, item);
                     return true;
@@ -475,10 +557,8 @@ namespace Hyena.Data.Sqlite
                 if (select_command == null) {
                     select_command = new HyenaSqliteCommand (
                         String.Format (
-                            "SELECT {0} FROM {1}{2}{3}",
-                            Select, From,
-                            (String.IsNullOrEmpty (Where) ? String.Empty : " WHERE "),
-                            Where
+                            "SELECT {0} FROM {1} WHERE {2}",
+                            Select, From, String.IsNullOrEmpty (Where) ? "1=1" : Where
                         )
                     );
                 }
@@ -518,6 +598,17 @@ namespace Hyena.Data.Sqlite
             }
         }
         
+        protected virtual HyenaSqliteCommand DeleteCommand {
+            get {
+                if (delete_command == null) {
+                    delete_command = new HyenaSqliteCommand (String.Format (
+                        "DELETE FROM {0} WHERE {1} IN (?)", TableName, PrimaryKey
+                    ));
+                }
+                return delete_command;
+            }
+        }
+        
         public virtual string Select {
             get {
                 if (select == null) {
