@@ -277,7 +277,7 @@ namespace Migo.Syndication
         private FeedDownloadStatus download_status;
         public FeedDownloadStatus DownloadStatus { 
             get { return download_status; }
-            set { download_status = value; Manager.OnFeedsChanged (); }
+            set { download_status = value; }
         }
         
         [DatabaseColumn("IsSubscribed")]
@@ -543,6 +543,11 @@ namespace Migo.Syndication
         }
 
         public void Save ()
+        {
+            Save (true);
+        }
+
+        public void Save (bool notify)
         {
             Provider.Save (this);
             
@@ -550,7 +555,9 @@ namespace Migo.Syndication
                 CheckForItemsToDownload ();
             }
 
-            Manager.OnFeedsChanged ();
+            if (notify) {
+                Manager.OnFeedsChanged ();
+            }
         }
         
         private void CheckForItemsToDownload ()
